/* eslint-disable no-magic-numbers */
import { ActionReducerMapBuilder, PayloadAction } from '@reduxjs/toolkit';
import { getLayers } from '@/redux/layers/layers.thunks';
import { LayersState } from '@/redux/layers/layers.types';

export const getLayersReducer = (builder: ActionReducerMapBuilder<LayersState>): void => {
  builder.addCase(getLayers.pending, state => {
    state.loading = 'pending';
  });
  builder.addCase(getLayers.fulfilled, (state, action) => {
    state.data = action.payload || {
      layers: [],
      layersVisibility: {},
    };
    state.loading = 'succeeded';
  });
  builder.addCase(getLayers.rejected, state => {
    state.loading = 'failed';
  });
};

export const setLayerVisibilityReducer = (
  state: LayersState,
  action: PayloadAction<{ visible: boolean; layerId: string }>,
): void => {
  const { payload } = action;
  if (state.data && state.data.layersVisibility[payload.layerId] !== undefined) {
    state.data.layersVisibility[payload.layerId] = payload.visible;
  }
};
