import { apiPath } from '@/redux/apiPath';
import { validateDataUsingZodSchema } from '@/utils/validateDataUsingZodSchema';
import { createAsyncThunk } from '@reduxjs/toolkit';
import { ThunkConfig } from '@/types/store';
import { getError } from '@/utils/error-report/getError';
import { axiosInstanceNewAPI } from '@/services/api/utils/axiosInstance';
import { ModelElements } from '@/types/models';
import { MODEL_ELEMENTS_FETCHING_ERROR_PREFIX } from '@/redux/modelElements/modelElements.constants';
import { modelElementSchema } from '@/models/modelElementSchema';
import { pageableSchema } from '@/models/pageableSchema';

export const getModelElements = createAsyncThunk<ModelElements | undefined, number, ThunkConfig>(
  'vectorMap/getModelElements',
  async (modelId: number) => {
    try {
      const response = await axiosInstanceNewAPI.get<ModelElements>(
        apiPath.getModelElements(modelId),
      );
      const isDataValid = validateDataUsingZodSchema(
        response.data,
        pageableSchema(modelElementSchema),
      );
      return isDataValid ? response.data : undefined;
    } catch (error) {
      return Promise.reject(getError({ error, prefix: MODEL_ELEMENTS_FETCHING_ERROR_PREFIX }));
    }
  },
);
