import { apiPath } from '@/redux/apiPath';
import {
  ToolkitStoreWithSingleSlice,
  createStoreInstanceUsingSliceReducer,
} from '@/utils/createStoreInstanceUsingSliceReducer';
import { mockNetworkNewAPIResponse } from '@/utils/mockNetworkResponse';
import { HttpStatusCode } from 'axios';
import { bioShapesFixture } from '@/models/fixtures/bioShapesFixture';
import { ShapesState } from '@/redux/shapes/shapes.types';
import { lineTypesFixture } from '@/models/fixtures/lineTypesFixture';
import { arrowTypesFixture } from '@/models/fixtures/arrowTypesFixture';
import shapesReducer from './shapes.slice';
import { getArrowTypes, getLineTypes, getShapes } from './shapes.thunks';

const mockedAxiosClient = mockNetworkNewAPIResponse();

describe('shapes thunks', () => {
  let store = {} as ToolkitStoreWithSingleSlice<ShapesState>;
  beforeEach(() => {
    store = createStoreInstanceUsingSliceReducer('shapes', shapesReducer);
  });

  describe('getShapes', () => {
    it('should return data when data response from API is valid', async () => {
      mockedAxiosClient.onGet(apiPath.getShapes()).reply(HttpStatusCode.Ok, bioShapesFixture);

      const { payload } = await store.dispatch(getShapes());
      expect(payload).toEqual(bioShapesFixture);
    });

    it('should return undefined when data response from API is not valid ', async () => {
      mockedAxiosClient
        .onGet(apiPath.getShapes())
        .reply(HttpStatusCode.Ok, { randomProperty: 'randomValue' });

      const { payload } = await store.dispatch(getShapes());
      expect(payload).toEqual(undefined);
    });
  });

  describe('getLineTypes', () => {
    it('should return data when data response from API is valid', async () => {
      mockedAxiosClient.onGet(apiPath.getLineTypes()).reply(HttpStatusCode.Ok, lineTypesFixture);

      const { payload } = await store.dispatch(getLineTypes());
      expect(payload).toEqual(lineTypesFixture);
    });

    it('should return undefined when data response from API is not valid ', async () => {
      mockedAxiosClient
        .onGet(apiPath.getLineTypes())
        .reply(HttpStatusCode.Ok, { randomProperty: 'randomValue' });

      const { payload } = await store.dispatch(getLineTypes());
      expect(payload).toEqual(undefined);
    });
  });

  describe('getArrowTypes', () => {
    it('should return data when data response from API is valid', async () => {
      mockedAxiosClient.onGet(apiPath.getArrowTypes()).reply(HttpStatusCode.Ok, arrowTypesFixture);

      const { payload } = await store.dispatch(getArrowTypes());
      expect(payload).toEqual(arrowTypesFixture);
    });

    it('should return undefined when data response from API is not valid ', async () => {
      mockedAxiosClient
        .onGet(apiPath.getArrowTypes())
        .reply(HttpStatusCode.Ok, { randomProperty: 'randomValue' });

      const { payload } = await store.dispatch(getArrowTypes());
      expect(payload).toEqual(undefined);
    });
  });
});
